TBF = 253   # Taxa Básica Financeira
TJLP = 256  # Taxa de juros - TJLP
TR = 226    # Taxa Referencial

SELIC = 11             # Taxa de juros - Selic
SELIC_ACUM_MES = 4390  # Taxa de juros - Selic acumulada no mês
SELIC_META = 432      # Taxa de juros - Meta Selic definida pelo Copom

CDI = 12  # Taxa de Juros - CDI

IGPDI = 190   # Índice geral de preços-disponibilidade interna (IGP-DI)
IGP10 = 7447  # Índice Geral de Preços-10 (IGP-10)
IPA = 7450    # Índice de Preços por Atacado-Mercado (IPA-M)
IPC = 191     # Índice de preços ao consumidor-Brasil (IPC-Br)
IPCA = 433    # Índice nacional de preços ao consumidor-amplo (IPCA)
INCC = 192    # Índice nacional de custo da construção (INCC)

OURO = 4  # Ouro BM&F - grama

BOVESPA_INDICE = 7              # Bovespa - índice
BOVESPA_VOLUME = 8              # Bovespa - Volume
BOVESPA_QTD_LISTADAS = 7850     # Quantidade de companhias listadas na Bovespa
BOVESPA_VALOR_LISTADAS = 7849	# Valor das empresas listadas na Bovespa
DOWN_JONES = 7809               # Dow Jones NYSE - índice
NASDAQ = 7810                   # Nasdaq - índice

PIB_RS_CORRENTE = 1207  # Produto interno bruto em R$ correntes
PIB_VAR_PERC = 7326     # Produto Interno Bruto - Tx. variação real no ano
POUPANCA_I = 25         # Depósitos poupança até 03.05.12 - Rentab. no período
POUPANCA_II = 195       # Depósitos poupança após 04.05.12 - Rentab. no período



import os

from jinja2 import BaseLoader, Environment
from lxml import etree
import pandas as pd
import requests

# Template XML para request ao Webservice do SGS
soapenv = 'http://schemas.xmlsoap.org/soap/envelope/'
xsd = 'http://www.w3.org/2001/XMLSchema'
xsi = 'http://www.w3.org/2001/XMLSchema-instance'

tpl = '''<?xml version="1.0" encoding="utf-8"?>
       <soapenv:Envelope xmlns:soapenv="{}" xmlns:xsd="{}" xmlns:xsi="{}">
           <soapenv:Body>
               <{{{{ method }}}} xmlns="https://www3.bcb.gov.br/wssgs/services/FachadaWSSGS">
                   {{% for p in params %}}
                       {{% if p == "codigosSeries" %}}
                           <{{{{ p }}}}><item>{{{{ params[p] }}}}</item></{{{{ p }}}}>
                       {{% else %}}
                           <{{{{ p }}}}>{{{{ params[p] }}}}</{{{{ p }}}}>
                       {{% endif %}}
                   {{% endfor %}}
              </{{{{ method }}}}>
           </soapenv:Body>
       </soapenv:Envelope>'''.format(soapenv, xsd, xsi)


def parse_data(data):
    dmy = data.split('/')
    fill = [2, 2, 4]
    dt = '/'.join([i.zfill(fill[n]) for n, i in enumerate(dmy)])
    return dt


class SGS():
    ''' Consulta séries temporais no SGS - Sistema Gerenciador de Séries
        Temporais do Banco Central do Brasil '''
    def __init__(self):
        pass

    def load_template(self, tpl_path='body_template.j2'):
        ''' Carrega e preeche o template do corpo do request ao webservice
            do SGS '''
        path, filename = os.path.split(tpl_path)
        template = Environment(loader=BaseLoader).from_string(tpl)
        return template

    def requests_wssgs(self, method, params):
        ''' Envia o request ao webservise do SGS '''
        URL_WEBSERVICE = 'https://www3.bcb.gov.br/wssgs/services/FachadaWSSGS'
        template = self.load_template()
        body = template.render(method=method, params=params)
        url = '{}?method={}'.format(URL_WEBSERVICE, method)
        headers = {'soapAction': '{}/{}'.format(URL_WEBSERVICE, method)}
        response = requests.post(url, headers=headers, data=body).content
        return response

    def get_valores_series(self, codigo_serie, data_inicio, data_fim):
        ''' Solicita uma série temporal ao SGS.

            Parâmetros:
            * codigo_serie(int): código da série
            * data_inicio(str): data de inicio no format dd/mm/yyyy
            * data_fim(str): data de fim no format dd/mm/yyyy

            Retorna dataframe contendo os valores da série temporal.
        '''

        method = 'getValoresSeriesXML'
        params = {'codigosSeries': codigo_serie,
                  'dataInicio': data_inicio,
                  'dataFim': data_fim}

        wssg_response = self.requests_wssgs(method, params)
        if 'Value(s) not found' in wssg_response.decode():
            msg = ('Valores não encontrados.'
                   ' Verifique o código da série e a data de vigência.')
            raise ValueError(msg)
        root = etree.fromstring(wssg_response)
        xml_return = root.xpath('// getValoresSeriesXMLReturn')[0]
        serie = etree.fromstring(xml_return.text.encode('ISO-8859-1'))[0]
        colum_names = [i.tag for i in serie[0]]
        serie_temporal = []
        for item in serie:
            values = []
            for coluna in item:
                val = coluna.text
                if coluna.tag.startswith('DATA'):
                    val = parse_data(coluna.text)
                values.append(val)
            serie_temporal.append(values)
        df = pd.DataFrame(serie_temporal, columns=colum_names)
        for col in df:
            if col.startswith('DATA'):
                df[col] = pd.to_datetime(df[col], dayfirst=True)
        if 'BLOQUEADO' in df.columns:
            del df['BLOQUEADO']
        return df



from distutils.core import setup

setup(
    name = 'sgs',
    packages = ['sgs'],
    version = '1.2',  
    description = 'Python wrapper para o webservice do SGS - Sistema Gerenciador de Series Temporais do Banco Central do Brasil.',
    author = 'Rafael Alves Ribeiro',
    author_email = 'rafael.alves.ribeiro@gmail.com',
    url = 'https://github.com/rafpyprog/sgs.git',
    classifiers = [
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',

        'Topic :: Scientific/Engineering :: Information Analysis',
    ],
)
